<?php

class OpenpayManager {
  public static function getMid()             { return OPENPAY_MID; }
  public static function getDigestKey1()      { return OPENPAY_KEY1; }
  public static function getDigestKey2()      { return OPENPAY_KEY2; }
  public static function getAccessKey()       { return OPENPAY_ACCESS_KEY; }

  public static function getInterPayUrl()     { return OPENPAY_INTER_PAY_URL; }
  public static function getApiPayUrl()       { return OPENPAY_API_PAY_URL; }
  public static function getApiTxInquiryUrl() { return OPENPAY_API_TX_INQUIRY_URL; }
  public static function getApiTxSearchUrl()  { return OPENPAY_API_TX_SEARCH_URL; }

  public static function getModeList()     { return array( 0, 1, 2 ); }
  public static function getPaymethodList(){ return array( 0, 1, 2, 4, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18 ); }
  public static function getIidList()      { return array( 0, 1, 3, 6, 12, 24, 36 ); }
  public static function getCharsetList()  { return array( 'UTF-8', 'BIG5', 'GB2312' ); }
  public static function getLanguageList() { return array( 'tchinese','schinese','english' ); }
  public static function getXtypeList()    { return array( 1, 2 ); }

  private static function _doHttpPost( $action_url, $action_param ) {
    return OpenpayManager::_doHttpPostByCurl( $action_url, $action_param );
  }

  private static function _doHttpPostByCurl( $action_url, $action_param ) {
    $ch = curl_init();
    //set the url, number of POST vars, POST data
    curl_setopt($ch, CURLOPT_URL, $action_url );
    curl_setopt($ch, CURLOPT_FAILONERROR, 1 );
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1 );
    curl_setopt($ch, CURLOPT_CAINFO, OPENPAY_SSL_CERT_FILE );
    curl_setopt($ch, CURLOPT_POST, 1 );
    curl_setopt($ch, CURLOPT_POSTFIELDS, $action_param );
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1 );
    $result = curl_exec($ch);
    $err_msg = curl_errno($ch) == 0 ? null : ('CURL '.curl_errno($ch).' - '.curl_error($ch));
    curl_close($ch);

    if( $err_msg !== null ) {
      throw new Exception( $err_msg );
    }

    return $result;
  }

  /**
   *  系統整合付款, 產生付款要求 html form 資訊.
   */
  public static function getInterPayRequest( $txid, $amount, $mode, $select_paymethod, $prefer_paymethod, $iid, $expire_date, $return_url, $charset, $language, $description_tchinese, $description_schinese, $description_english, $note_tchinese, $note_schinese, $note_english, $cart_type, $cid, $cname, $caddress, $ctel, $cemail, $xtype, $xname, $xaddress, $xtel, $pcsccod_recv_store_id ) {

    $txid = trim($txid);
    if( $amount < 1 )
      throw new Exception( "amount must >= 1." );
    if( $amount > 150000 )
      throw new Exception( "amount must <= 150000." );
    if( ceil($amount) != $amount )
      throw new Exception( "amount must be integer number." );
    $mode = (int)$mode;
    if( ! in_array($mode, OpenpayManager::getModeList()))
      throw new Exception( "mode value not available." );
    $select_paymethod = (int)$select_paymethod;
    if( ! in_array($select_paymethod, OpenpayManager::getPaymethodList()))
      throw new Exception( "select_paymethod value not available." );
    $prefer_paymethod = (int)$prefer_paymethod;
    if( ! in_array($prefer_paymethod, OpenpayManager::getPaymethodList()))
      throw new Exception( "prefer_paymethod value not available." );
    $iid = (int)$iid;
    if( ! in_array($iid, OpenpayManager::getIidList()))
      throw new Exception( "iid value not available." );
    $expire_date = trim($expire_date);
    if( (strlen($expire_date) > 0) && (strtotime($expire_date) < strtotime(date("Y-m-d"))) )
      throw new Exception( "expire_date must >= today." );
    $return_url = trim($return_url);
    $charset = trim($charset);
    if( ! in_array($charset, OpenpayManager::getCharsetList()))
      throw new Exception( "charset value not available." );
    $language = trim($language);
    if( ! in_array($language, OpenpayManager::getLanguageList()))
      throw new Exception( "language value not available." );
    $description_tchinese  = trim($description_tchinese);
    $description_schinese  = trim($description_schinese);
    $description_english   = trim($description_english);
    $note_tchinese         = trim($note_tchinese);
    $note_schinese         = trim($note_schinese);
    $note_english          = trim($note_english);
    $cart_type             = trim($cart_type);
    $cid                   = trim($cid);
    $cname                 = trim($cname);
    $caddress              = trim($caddress);
    $ctel                  = trim($ctel);
    $cemail                = trim($cemail);
    $xtype                 = (int) $xtype;
    if( ! in_array($xtype, OpenpayManager::getXtypeList()))
      throw new Exception( "xtype value not available." );
    $xname                 = trim($xname);
    $xaddress              = trim($xaddress);
    $xtel                  = trim($xtel);
    $pcsccod_recv_store_id = trim($pcsccod_recv_store_id);

    $action_url = OpenpayManager::getInterPayUrl();
    $action_param = array();
    $action_param['version']               = "2.1";
    $action_param['mid']                   = OpenpayManager::getMid();
    $action_param['txid']                  = $txid;
    $action_param['amount']                = $amount;
    $action_param['verify']                = md5(implode( "|", array(OpenpayManager::getDigestKey1(), $action_param['mid'], $action_param['txid'], $action_param['amount'], OpenpayManager::getDigestKey2())));
    $action_param['mode']                  = $mode;
    $action_param['select_paymethod']      = $select_paymethod;
    $action_param['prefer_paymethod']      = $prefer_paymethod;
    $action_param['iid']                   = $iid;
    $action_param['expire_date']           = $expire_date;
    $action_param['return_url']            = $return_url;
    $action_param['charset']               = $charset;
    $action_param['language']              = $language;
    $action_param['description_tchinese']  = $description_tchinese;
    $action_param['description_schinese']  = $description_schinese;
    $action_param['description_english']   = $description_english;
    $action_param['note_tchinese']         = $note_tchinese;
    $action_param['note_schinese']         = $note_schinese;
    $action_param['note_english']          = $note_english;
    $action_param['cart_type']             = $cart_type;
    $action_param['cid']                   = $cid;
    $action_param['cname']                 = $cname;
    $action_param['caddress']              = $caddress;
    $action_param['ctel']                  = $ctel;
    $action_param['cemail']                = $cemail;
    $action_param['xtype']                 = $xtype;
    $action_param['xname']                 = $xname;
    $action_param['xaddress']              = $xaddress;
    $action_param['xtel']                  = $xtel;
    $action_param['pcsccod_recv_store_id'] = $pcsccod_recv_store_id;

    return array( 'action_url' => $action_url, 'action_param' => $action_param );
  }

  /**
   *  系統整合付款, 瀏覽器傳回的付款結果資訊檢查.
   */
  public static function verifyInterPayResponse( $txid, $amount, $pay_type, $status, $tid, $verify ) {
    $digest_verify = md5(implode( "|", array(OpenpayManager::getDigestKey1(), $txid, $amount, $pay_type, $status, $tid, OpenpayManager::getDigestKey2())));

    if( $digest_verify !== $verify )
      return false;

    return true;
  }

  /**
   *  系統取單付款, 透過 API 取得離線付款開單的付款資訊
   */
  public static function doApiPayAction( $txid, $amount, $expire_date, $is_cstore, $is_fami, $is_ibon, $is_pcsccod, $is_vaccount, $encoding, $description, $cname, $caddress, $ctel, $xname, $xtel, $pcsccod_recv_store_id ) {

    $txid = trim($txid);
    if( $amount < 1 )
      throw new Exception( "amount must >= 1." );
    if( $amount > 150000 )
      throw new Exception( "amount must <= 150000." );
    if( ceil($amount) != $amount )
      throw new Exception( "amount must be integer number." );
    $expire_date = trim($expire_date);
    if( (strlen($expire_date) > 0) && (strtotime($expire_date) < strtotime(date("Y-m-d"))) )
      throw new Exception( "expire_date must >= today." );
    $is_cstore   = ((int) $is_cstore) ? 1 : 0;
    $is_fami     = ((int) $is_fami) ? 1 : 0;
    $is_ibon     = ((int) $is_ibon) ? 1 : 0;
    $is_pcsccod  = ((int) $is_pcsccod) ? 1 : 0;
    $is_vaccount = ((int) $is_vaccount) ? 1 : 0;
    $encoding = trim($encoding);
    if( ! in_array($encoding, OpenpayManager::getCharsetList()))
      throw new Exception( "encoding value not available." );
    $description = trim($description);
    $cname       = trim($cname);
    $caddress    = trim($caddress);
    $ctel        = trim($ctel);
    $xname       = trim($xname);
    $xtel        = trim($xtel);
    $pcsccod_recv_store_id = trim($pcsccod_recv_store_id);

    $action_url = OpenpayManager::getApiPayUrl();
    $action_param = array();
    $action_param['version']               = "2.1";
    $action_param['mid']                   = OpenpayManager::getMid();
    $action_param['access_key']            = OpenpayManager::getAccessKey();
    $action_param['txid']                  = $txid;
    $action_param['amount']                = $amount;
    $action_param['expire_date']           = $expire_date;
    $action_param['is_cstore']             = $is_cstore;
    $action_param['is_fami']               = $is_fami;
    $action_param['is_ibon']               = $is_ibon;
    $action_param['is_pcsccod']            = $is_pcsccod;
    $action_param['is_vaccount']           = $is_vaccount;
    $action_param['encoding']              = $encoding;
    $action_param['description']           = $description;
    $action_param['cname']                 = $cname;
    $action_param['caddress']              = $caddress;
    $action_param['ctel']                  = $ctel;
    $action_param['xname']                 = $xname;
    $action_param['xtel']                  = $xtel;
    $action_param['pcsccod_recv_store_id'] = $pcsccod_recv_store_id;

    $action_response = OpenpayManager::_doHttpPost( $action_url, $action_param );

    return json_decode($action_response, 1);
  }

  /**
   *  付款狀態查詢, 透過 API 取得指定 "訂單編號" 代收交易的付款狀態
   */
  public static function verifyNotifyFundin( $notify_param ) {
    if( $notify_param['access_key'] != OpenpayManager::getAccessKey() ) {
      // log error to application log
      return false;
    }

    $digest_text  = implode( "|", array(OpenpayManager::getDigestKey1(), $notify_param['txid'], $notify_param['amount'], $notify_param['pay_type'], $notify_param['status'], $notify_param['tid'], OpenpayManager::getDigestKey2()));
    $digest_value = md5( $digest_text );
    if( $digest_value !== $notify_param['verify'] ) {
      // log error to application log
      return false;
    }

    return true;
  }

  /**
   *  付款狀態查詢, 透過 API 取得指定 "訂單編號" 代收交易的付款狀態
   */
  public static function doApiTxInquiry( $order_no ) {
    if( strlen($order_no) <= 0 ) {
      throw new Exception( "ERROR_LIB: order_no must not empty" );
    }

    $req_verify_text = OpenpayManager::getAccessKey()."|".OpenpayManager::getMid()."|".$order_no;
    $req_verify      = md5( $req_verify_text );

    $action_url = OpenpayManager::getApiTxInquiryUrl();
    $action_param = array();
    $action_param['mid']    = OpenpayManager::getMid();
    $action_param['txid']   = $order_no;
    $action_param['verify'] = $req_verify;

    $action_res_jstr = OpenpayManager::_doHttpPost( $action_url, $action_param );
    $action_res      = json_decode($action_res_jstr, 1);

    $res_verify_text = OpenpayManager::getAccessKey()."|".$action_res['status']."|".$action_res['res_jstr'];
    $res_verify      = md5( $res_verify_text );

    if( $res_verify !== $action_res['verify'] ) {
      throw new Exception( "ERROR_LIB: response verify error" );
    }

    $res_param = json_decode($action_res['res_jstr'], 1);

    if( $res_param['status'] != 101 ) {
      throw new Exception( "ERROR_API: ".$res_param['status_desc'] );
    }

    return $res_param;
  }

  /**
   *  付款狀態查詢, 透過 API 查詢指定日期區間的已付款交易結算狀態.
   */
  public static function doApiTxSearch( $fundin_date_begin, $fundin_date_end ) {
    if( strlen($fundin_date_begin) <= 0 ) {
      throw new Exception( "ERROR_LIB: begin of fundin date must not empty" );
    }
    if( strlen($fundin_date_end) <= 0 ) {
      throw new Exception( "ERROR_LIB: end of fundin date must not empty" );
    }
    if( strtotime($fundin_date_begin) < 1 ) {
      throw new Exception( "ERROR_LIB: begin of fundin date must be a date" );
    }
    if( strtotime($fundin_date_end) < 1 ) {
      throw new Exception( "ERROR_LIB: end of fundin date must be a date" );
    }

    $fundin_date_begin = date("Y-m-d", strtotime($fundin_date_begin));
    $fundin_date_end   = date("Y-m-d", strtotime($fundin_date_end));

    $req_verify_text = OpenpayManager::getAccessKey()."|".OpenpayManager::getMid()."|".$fundin_date_begin."|".$fundin_date_end;
    $req_verify      = md5( $req_verify_text );

    $action_url = OpenpayManager::getApiTxSearchUrl();
    $action_param = array();
    $action_param['mid']    = OpenpayManager::getMid();
    $action_param['fundin_date_begin'] = $fundin_date_begin;
    $action_param['fundin_date_end']   = $fundin_date_end;
    $action_param['verify']            = $req_verify;

    $action_res_jstr = OpenpayManager::_doHttpPost( $action_url, $action_param );
    $action_res      = json_decode($action_res_jstr, 1);

    if( $action_res['status'] < 100 ) {
      throw new Exception( "ERROR_API: ".$action_res['status_desc'] );
    }

    $res_verify_text = OpenpayManager::getAccessKey()."|".$action_res['status']."|".$action_res['res_jstr'];
    $res_verify      = md5( $res_verify_text );

    if( $res_verify !== $action_res['verify'] ) {
      throw new Exception( "ERROR_LIB: response verify error" );
    }

    $res_param = json_decode($action_res['res_jstr'], 1);

    return $res_param;
  }

}

